/*
 * Copyright (c) 2012, IDM
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 *     * Neither the name of the IDM nor the names of its contributors may be used to endorse or
 *       promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>

#import "SkPublishAPI.h"
#import "RequestHandler.h"
#import "MyRequestHandler.h"

#import "JSONUtils.h"
#import "SBJsonWriter.h"

int main(int argc, char *argv[])
{
    if (argc < 3) {
        NSLog(@"need baseurl and accesskey in parameters");
    }

    NSString *buildPath = [NSString stringWithUTF8String:argv[0]];
    NSString *baseUrl = [NSString stringWithUTF8String:argv[1]];
    NSMutableString *url = [[NSMutableString alloc] initWithString:baseUrl];
    [url appendString: @"/api/v1/"];
    NSString *accessKey = [NSString stringWithUTF8String:argv[2]];

    NSLog(@"arg0 (BUILDPATH) : %@", buildPath);
    NSLog(@"arg1 (URL) : %@", url);
    NSLog(@"arg2 (ACCESSKEY) : %@", accessKey);


    SkPublishAPI *api = [[SkPublishAPI alloc] initWithUrlAndAcessKey:url :accessKey];

    MyRequestHandler *requestHandler = [[MyRequestHandler alloc] init];
    [api setRequestHandler: requestHandler];

    JSONUtils *jsonUtils = [[JSONUtils alloc] init];
    NSString *result;


    NSLog(@"*** Dictionaries");

    result = [api getDictionaries];
    NSLog(@"%@", result);

    NSDictionary *firstElement = [jsonUtils getFirstResult: result];
    if(firstElement == NULL) {
        NSLog(@"No dictionary found.");
        return 1;
    }
    //NSLog(@"First Dictionary: %@", firstElement);

    if(![firstElement objectForKey: @"dictionaryCode"]){
        SBJsonWriter *jsonWriter = [[SBJsonWriter alloc] init];
        NSMutableString *error = [[NSMutableString alloc] initWithString: @"dictionaryCode property was expected in the JSON object : "];    
        NSString *dictRespStr = [jsonWriter stringWithObject: firstElement];
        [error appendString: dictRespStr];
        NSLog(@"%@", error);
        return 1;
    }

    NSString *dictCode = [firstElement objectForKey: @"dictionaryCode"];    
    //NSLog(@"dictionaryCode = %@", dictCode);


    NSLog(@"*** Search");
    NSLog(@"*** Result list");

    result = [api search: dictCode : @"ca" : [NSNumber numberWithInt:1] : [NSNumber numberWithInt:1]];
    NSLog(@"%@", result);


    NSLog(@"*** Spell checking");

    result = [api didYouMean: dictCode : @"dorg": [NSNumber numberWithInt:3]];
    NSLog(@"%@", result);


    NSLog(@"*** Best matching");

    result = [api searchFirst: dictCode : @"ca" : @"html"];
    NSLog(@"%@", result);

    firstElement = [jsonUtils getUniqueResult: result];
    if(firstElement == NULL) {
        NSLog(@"No best matching found.");
        return 1;
    }
    //NSLog(@"First Best Matching: %@", firstElement);

    if(![firstElement objectForKey: @"entryId"]){
        SBJsonWriter *jsonWriter = [[SBJsonWriter alloc] init];
        NSMutableString *error = [[NSMutableString alloc] initWithString: @"entryId property was expected in the JSON object : "];    
        NSString *dictRespStr = [jsonWriter stringWithObject: firstElement];
        [error appendString: dictRespStr];
        NSLog(@"%@", error);
        return 1;
    }

    NSString *entryId = [firstElement objectForKey: @"entryId"];

    NSLog(@"*** Nearby Entries");

    result = [api getNearbyEntries: dictCode : entryId : [NSNumber numberWithInt:3]];
    NSLog(@"%@", result);


    return 0;
}

